program ReadUSB

  use ISO_C_binding

! Comment out the following if using a compiler that does not support
! the Fortran 2003 Get_Command_Argument intrinsic subroutine, and
! that provides Getarg.

  intrinsic Get_Command_Argument

  interface
    ! Check whether FD has input ready, and read it into BUF if so.
    ! Function result is negative if things go haywire.
    ! Function result is zero if nothing arrive within Tmout milliseconds.
    ! Else the function result is 1.
    function Check_IO ( FD, Tmout, Buf, Bufsize, Bytes_Read ) bind(C)
      use ISO_C_binding
      integer(c_int) :: Check_IO
      integer(c_int), intent(in), value :: FD      ! File handle
      integer(c_int), intent(in), value :: Tmout   ! Timeout, milliseconds
      character(kind=c_char), intent(out) :: Buf
      integer(c_int), intent(in), value :: Bufsize ! Size of Buf
      integer(c_int), intent(out) :: Bytes_Read    ! How many bytes read
    end function Check_IO

    subroutine Close_Port ( FD ) bind(C)
      use ISO_C_binding
      integer(c_int), intent(in), value :: FD      ! File handle
    end subroutine Close_Port

    function Open_Port ( Port ) bind(c)
      use ISO_C_binding
      integer(c_int) :: Open_Port
      character(kind=c_char), intent(in) :: Port
    end function Open_Port

    ! Send N characters from Line to the port with FD handle.
    ! If the result value is negative something went wrong, and the
    ! function will have printed an error message.
    function Send_Line ( FD, Line, N ) bind(c)
      use ISO_C_binding
      integer(c_int) :: Send_Line
      integer(c_int), intent(in), value :: FD       ! File handle
      character(kind=c_char), intent(in) :: Line    ! Line to send
      integer(c_int), intent(in), value ::N         ! Number of characters to send
    end function Send_Line

  end interface

  integer(c_int), parameter :: Bsz = 255      ! Buf size
  character(len=bsz,kind=c_char) :: Buf       ! Of returned data from the port
  integer(c_int) :: Bytes_Read
  integer(c_int) :: FD      ! File descriptor for the serial port
  integer :: C, I, J, N
  character(len=bsz,kind=c_char) :: Line      ! To send to the port
  character(len=*,kind=c_char), parameter :: DefaultPort = '/dev/ttyACM0'
  character(len=bsz) :: Port = ''
  integer(c_int), parameter :: Tmout = 10     ! Milliseconds time out for read

  call get_command_argument ( 1, port )
  if ( port == '' ) port = defaultPort
  fd = open_port ( trim(port) // char(0) )
  if ( fd < 0 ) then
    write ( *, '(a,i0)' ) &
      & 'Unable to setup port ' // trim(port) // '; status =', fd
    stop
  end if

  ! Read from the serial port until it says it doesn't have anything
  ! more to send
  do while ( check_io ( fd, tmout, buf, bsz, bytes_read ) > 0 )
  end do

  do
    write ( *, '(a)', advance='no' ) 'Ready for input: '
    ! Read a line to send to the device,  Quit on EOF or line(1:1) == 'z'
    read ( *, '(a)', end=9 ) line
    if ( line(1:1) == 'z' ) exit
    c = len_trim(line)
    ! Send the line to the device. It wants the line to end with LF CR.
    line(c+1:c+2) = char(10) // char(13)
    if ( send_line ( fd, line, c+2 ) < 0 ) stop
    ! Read the lines returned by the device
    ! Print the ones that aren't empty and aren't what was sent,
    ! ignoring a prompt (>) at the beginning of the line
    bytes_read = 0
    do while ( check_io ( fd, tmout, buf(bytes_read+1:), bsz-bytes_read, n ) > 0 )
      bytes_read = bytes_read + n
    end do
    i = 1
    do while ( i <= bytes_read )
      do i = i, bytes_read ! Skip leading control characters and '>'
        if ( buf(i:i) /= '>' .and. ichar(buf(i:i)) > 31 ) exit
      end do
      if ( i > bytes_read ) exit
      do j = i, bytes_read ! Look for a control character
        if ( ichar(buf(j:j)) < 32 ) exit
      end do
      ! Print the reply if it's not empty and not what was input
      if ( j >= i .and. buf(i:j-1) /= line(:c) ) write ( *, '(a)' ) buf(i:j-1)
      i = j + 1
    end do
  end do

9 continue

  call close_port ( fd )

! Un-comment the following if using a compiler that does not support
! the Fortran 2003 Get_Command_Argument intrinsic subroutine, and
! that provides Getarg.  On HP-UX Fortran, it is necessary to add 1
! to ArgNum: argument 1 is the command, while on (all?) other systems,
! argument 0 is the command.

! contains
! 
!   subroutine Get_Command_Argument ( ArgNum, ArgVal )
!     use ISO_C_binding
!     integer, intent(in) :: ArgNum
!     character(len=*), intent(out) :: ArgVal
!     call Getarg ( argNum, value=argVal )
!   end subroutine Getarg

end program ReadUSB
